# -*- coding: utf-8 -*-

import hashlib
import hmac
import six

try:
    compare_digest = hmac.compare_digest
except AttributeError:
    # Not secure when the length is supposed to be kept secret
    def compare_digest(a, b):
        if len(a) != len(b):
            return False
        return reduce(lambda x, y: x | y, [ord(x) ^ ord(y) for x, y in zip(a, b)]) == 0

def sign(secret, string_to_sign):
    return six.text_type(
        hmac.new(
                secret.encode('utf8'),
                string_to_sign.encode('utf8'),
                hashlib.sha256
            )
            .hexdigest()
        )

def verify(secret, string_to_sign, signature):
    return compare_digest(signature, sign(secret, string_to_sign))



# -*- coding: utf-8 -*-

import aiohttp
import asyncio

from pusher.http import process_response

class AsyncIOBackend:
    def __init__(self, config):
        """Adapter for the requests module.

        :param config:  pusher.Pusher object
        """
        self.config = config
        self.conn = aiohttp.TCPConnector()

    def send_request(self, request):
        method = request.method
        url = "%s%s" % (request.base_url, request.path)
        params = request.query_params
        data = request.body
        headers = request.headers

        response = yield from asyncio.wait_for(
            aiohttp.request(method, url, params=params, data=data, headers=headers, connector=self.conn),
            timeout=self.config.timeout
        )
        body = yield from response.read_and_close()
        return process_response(response.status, body.decode('utf8'))



# -*- coding: utf-8 -*-

from __future__ import (print_function, unicode_literals, absolute_import,
                        division)

import urlfetch
from pusher.http import process_response

class URLFetchBackend(object):
  """Adapter for the URLFetch Module. Necessary for using this library with Google
  App Engine"""

  def __init__(self, config, **options):
    self.config = config
    self.options = options
  
  def send_request(self, request):
    resp = urlfetch.fetch(
      url=request.url,
      headers=request.headers,
      method=request.method,
      data=request.body,
      deadline=self.config.timeout,
      **self.options
    )
    return process_response(resp.status_code, resp.content)